package cn.wise.sc.energy.power.plant.business.utils.poi;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.resource.ClassPathResource;
import cn.wise.sc.energy.power.plant.business.domain.AutoInspection;
import cn.wise.sc.energy.power.plant.business.repository.AutoInspectionRepository;
import cn.wise.sc.energy.power.plant.common.core.exception.ResponseEnum;
import com.alibaba.fastjson.JSONObject;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFHeader;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author shulidong
 * @since 2020/9/21 14:04
 */
@Service
public class DocxService {


    @Autowired
    AutoInspectionRepository autoInspectionRepository;
    private static final Logger LOGGER = LoggerFactory.getLogger(DocxService.class);

    public byte[] fillDocx() throws Exception {

        List<AutoInspection> list = autoInspectionRepository.findAutoInspectionsById("C0A001 - 1 - 1600756452620");
        if (CollectionUtil.isEmpty(list)) {
            ResponseEnum.BUSINESS_ERROR.newException("未找到相应的报告项");
        }
        final InputStream inputStream = new ClassPathResource("templates/" + "demo1.docx").getStream();
        XWPFDocument document = new XWPFDocument(inputStream);

        //添加页眉 需要固定的文字(中文标点)在模板上->{{:text:date:date:}}
        List<XWPFHeader> pageHeaders = document.getHeaderList();
        JSONObject original = new JSONObject();
        JSONObject date = new JSONObject();
        date.put("date",LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));
        original.put("date", date);
        for (int i = 0; i < pageHeaders.size(); i++) {
            XWPFHeader pageHeader = pageHeaders.get(i);
            List<XWPFParagraph> headerPara = pageHeader.getParagraphs();
            XWPFParagraph headerParagraph = headerPara.get(0);
            if (headerParagraph.getRuns().size() > 1) {
                PoiiUtiil.parseThisParagraph(headerParagraph,original);
            }
        }
        //获取所有的段落
        List<XWPFParagraph> paragraphs = document.getParagraphs();
        //把文档groupby
        List<XWPFParagraph> texts = new ArrayList<>();
        List<XWPFTable> tables = new ArrayList<>();
        for (XWPFParagraph paragraph : paragraphs) {
            String text = paragraph.getText();
            //只有这三种标签能够混用
            if (text.contains("{{:text") || text.contains("{{:for") || text.contains("{{:desc")) {
                texts.add(paragraph);
            }
        }
        for (XWPFTable table : document.getTables()) {
            if (table == null) {
                LOGGER.error("table is null, please stop!");
            }
            if (table.getText().contains("{{:table") || table.getText().contains("{{:text")) {
                tables.add(table);
            }
        }
        JSONObject allin = new JSONObject();

        JSONObject fuzhu = new JSONObject();
        fuzhu.put("MKW05CP001", 12.578);
        allin.put("fuzhu", fuzhu);


        JSONObject reportInfo = new JSONObject();
        reportInfo.put("username", "张三");
        reportInfo.put("a", "张三");
        reportInfo.put("date", LocalDateTime.now().format(DateTimeFormatter.BASIC_ISO_DATE).toString());
        allin.put("reportInfo", reportInfo);


        try {
            PoiiUtiil.paragraph(texts, allin);
            PoiiUtiil.table(tables, allin);
            //要求word在打开时提示更新域
            document.enforceUpdateFields();
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            FileOutputStream fos = new FileOutputStream("D:/"+ "生成文件.docx");
            document.write(fos);
            fos.flush();
            fos.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
